;******************************************************************************
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    I (Simon Wood) can be contacted via email - unilink@mungewell.org
;    or via post - 14 Kirklands Close, Baildon, West Yorkshire, England.
;
;******************************************************************************
;	Additional Info - Release version  1.8T_02
;		Modified Version : GNUnilink for MDX-5V101R
;			"Released under the GPL license, 
;					(c) H "Turboh" Ishimoto, June 2004"
;					   Email: ishimoto@bea.hi-ho.ne.jp
;******************************************************************************
; Functions for GNUnilink - these have to be placed at start of code as the 
; PIC12 series can only 'Call' to lower addresses
;******************************************************************************
; Lookup table for serial output - defines the output strings.
; Must not cross a page boundary!

;; --- Output messages are Removed by Turboh (Jan. 26 2004) ---

;******************************************************************************
; Send a Slave Break - tells the Master that you have something to tell them...
;
; In this function GNUnilink looks for a period when the data bus is low for 
; more than 6ms and then waits for it to be high for 2ms. Once this is seen it 
; knows were in the middle of an unused 'slave phase' and it then forces the 
; bus low for 4ms.

; NOTE - whilst GNUnilink is in this state it is unable to receive other 
; commands from the master

;; NOTE - Small but Big Change: -- "Two phase detection"    (T)
;; Modification:
;; 	At first, wait for 2.0ms of data-low, and after that, keep 
;; 	waiting more 3.5ms of data-low to start sending ISB.
;; 	If data-high is detected in the first phase, retry ISB.
;;	But, If data-high is detected in the second phase, go abort
;;	and memorize to retry in the next chance.
;; 	The next chances are
;;		(A)Ping-End(=Hello) detected  (case Display=1 or 2).
;;		(B)At the beginning of ReceiveBreak (case Display=3).
;;
;;      *NOTE - Meanings of Display are modified. See Display.asm

IssueSlaveBreak:
;; --- The first phase: Wait for 2.0ms-Low continues.
	movlw	20		;; Limit 2.0ms.  Adjusted Jan. 17 2004 (T)
	movwf	Count

BreakLoopP1:
	movlw	delay100us		; 100us 
	movwf	Count2

BreakLoopP2:
	btfsc	OUT_PORT, DataBit	; 
	goto	IssueSlaveBreak		; (Data High) Retry
	goto	BreakLoopP3		; (Data low) (NOP * 2)
BreakLoopP3:
	decfsz	Count2, 1		; Reduce Count2
	goto	BreakLoopP2

	decfsz	Count, 1		; Reduce Count 
	goto	BreakLoopP1

IssueSlaveBreak2:
;; --- The second phase: Wait for 3.5ms-Low continues.
	movlw	35			; 3.5ms (35 * 100us)
	movwf	Count

BreakLoop1:
	movlw	delay100us		; 100us 
	movwf	Count2

BreakLoop2:
	btfsc	OUT_PORT, ClockBit	;; TEST  ;; May 31 2004
					;; case ClockBit = High
	goto	ReserveISB		; Do Not Retry Continuously
	goto	BreakLoop3		; NOP

BreakLoop3:
	decfsz	Count2, 1		; Reduce Count2
	goto	BreakLoop2
	decfsz	Count, 1		; Reduce Count 
	goto	BreakLoop1

;; --- Wait for 2.0ms-High continues
BreakLoop4:
	movlw	20			; 2ms (20 * 100us)
	movwf	Count

BreakLoop5:
	movlw	delay100us
	movwf	Count2

BreakLoop6:
	btfss	OUT_PORT, DataBit	; Data high for 2ms 
	goto	BreakLoop4		; (go into 'Slave Phase')
	goto	BreakLoop7		; NOP

BreakLoop7:
	decfsz	Count2, 1		; Reduce Count2
	goto	BreakLoop6
	decfsz	Count, 1		; Reduce Count 
	goto	BreakLoop5

	IFDEF PIC_16
	bsf	STATUS, RP0		; Enable data pin as output
	bcf	OUT_ENA, DataBit
	bcf	STATUS, RP0
	ENDIF

;; --- Disable interrupt		;; Jan. 17 2004 (T)
	bcf	INTCON, GIE		;; Disable Global Interrupt

;; --- Force Data Low
	bcf	OUT_PORT, DataBit	; Force Data low.... 4ms
	movlw	30			;; Mod   3.0ms   Feb. 06 2004 (T)
	movwf	Count
BreakLoop9:
	movlw	delay100us
	movwf	Count2
BreakLoop10:
	goto	BreakLoop11		; 2 * NOP
BreakLoop11:
	goto	BreakLoop12		; 2 * NOP
BreakLoop12:
	decfsz	Count2, 1
	goto	BreakLoop10
	decfsz	Count, 1
	goto	BreakLoop9

	IFDEF PIC_16
	bsf	STATUS, RP0		; Tristate data pin
	bsf	OUT_ENA, DataBit
	bcf	STATUS, RP0
	ENDIF

;; --- Flag set for sending "Request to talk"	Apr. 19 2004 (T)

	bsf	NeedToTalk, 0		;; 

;; --- Enable interrupt			;; Feb. 7 2004 (T)
	bsf	INTCON, GIE		;; Enable Global Interrupt

	return

;******************************************************************************
; Receive a byte - received byte is placed in W register.
;
; Input is sampled between 1 cycle before and 3 cycles after Clock rising edge,
; depending on the asynchronous alignment with the Master

ReadByte:
	clrf	DataVal
	movlw	0x08
	movwf	Count
	bcf	STATUS, C		; Ensure carry bit is 0

ReadLoop1:
	rlf	DataVal, 1
	
ReadLoop2:
ReadLoop2a:
	btfss	OUT_PORT, ClockBit	; Wait for clock going high
	goto	ReadLoop2

	bcf	INTCON, GIE		;; Disable Global Interrupt

ReadLoop3:
	movf	OUT_PORT, 0		; Sample input

	btfsc	OUT_PORT, ClockBit	; Wait for clock going low
	goto	ReadLoop3

	movwf	Count2			; (inverted) input
	btfss	Count2, DataBit
	bsf	DataVal, 0
	
	decfsz	Count, 1		; Do next bit, if any left...
	goto	ReadLoop1

	bsf	INTCON, GIE		;; Enable Global Interrupt

	movf	DataVal, 0
	return

;******************************************************************************
; Send a byte - byte to output is in W register.
;
; This is the piece of code which is the most time critical.
; Worst case the output bit is set a maximum of 7 cycles after Clock goes High
; and held until the Clock goes high again
;
; this hopefully gives enough time for the master to register it.... a LOT 
; dodgy on slow systems.
;
; We may have to reverse the logic on this so that the output is set before the 
; Clock goes high and changed after the Clock goes low. This will improve the i
; set up, but reduce the hold time.

SendByte:
	movwf	DataVal			; Store byte to output

	addwf	Checksum, 1		; Compute Checksum

	IFDEF PIC_16
	bsf	STATUS, RP0		; Enable data pin
	bcf	OUT_ENA, DataBit
	bcf	STATUS, RP0
	ENDIF

	movlw	0x08
	movwf	Count

	bcf	INTCON, GIE		;; Disable Global Interrupt

SendLoop1:
	bcf	OUT_PORT, DataBit	; remember it's INVERTED
	nop				;; (T)

SendLoop2:
	btfss	OUT_PORT, ClockBit	; Wait for clock going high
	goto	SendLoop2
	btfss	DataVal, 7
	bsf	OUT_PORT, DataBit
	nop				;; (T)
	rlf	DataVal, 1

SendLoop3:
	btfsc	OUT_PORT, ClockBit	; Wait for clock going low
	goto	SendLoop3
	decfsz	Count, 1		; Do next bit, if any left...
	goto	SendLoop1
	goto	SendLoop4		; just a little data hold

SendLoop4:
	IFDEF PIC_16
	bsf	STATUS, RP0		; Tristate data pin
	bsf	OUT_ENA, DataBit
	bcf	STATUS, RP0
	ENDIF

	bsf	INTCON, GIE		;; Enable Global Interrupt
	return

;******************************************************************************
; Bit frig - works out which bit to set in the response to Master Poll
; W register is input of which stage you are on (0x00, 0x20, 0x30, 0x40 etc)
; and is returned with the byte to write (0x00 if wrong stage).

Bit_Frig:
	xorwf	MyBit, 0
	andlw	0xe0			; Strip off low bits

	btfsc	STATUS, Z		; Do we have a hit?
	goto	Bit_Frig_Hit

	movlw	0x00
	return

Bit_Frig_Hit:
	btfss	MyBit, 4		; Do we need to swap nibbles?
	goto	Bit_Frig_Swap

	movf	MyBit, 0
	andlw	0x0F
	return

Bit_Frig_Swap:
	swapf	MyBit, 0
	andlw	0xF0
	return

;******************************************************************************
; PoorTimeCounter	(T)
; Only in case MD mode, Although the second is expressed by BCD, 
; the minute is by Hex.    ...I don't know why..
Countup:
	call	Countup1		;; C=0 negative

	movf	Time0i, 0		;; Time0i -> W
	call	Diff_H2Tim		;;
	addwf	Time0i,	0		;; +result -> W
	movwf	Time0			;; set W ---> Time0

;;-- In case CD, The minute(Time1) is expressed by BCD.
	movf	MyGroup, 0		;; MyGroup -> W
	xorlw	0x30
	btfss	STATUS, Z		;;
	goto	M_BCD_Skip		;; case MD, Skip. (As it is.)

					;; case CD, convert Hex -> BCD
	movf	Time1i, 0		;; Time1i -> W
	call	Diff_H2Tim		;;
	addwf	Time1i,	0		;; +result -> W
;;-- In case CD, the minutes from 0 to 9 must be expressed "0xf0" to "0xf9".
	movwf	Time1iT			;; restore W -> Time1iT
	sublw	0x0f			;;  (0x0f - W)
	movf	Time1iT, 0		;; reload Time1iT -> W
	btfsc	STATUS,  C		;; 
	addlw	0xf0			;; (positive)  W(=Time1iT) + f0 -> W
	goto	M_BCD_Skip2

M_BCD_Skip:
	movf	Time1i, 0		;; Time1i -> W

M_BCD_Skip2:
	movwf	Time1			;; set W ---> Time1	;; 
	return

Countup1:
	clrf	Timep			;; set Timep=0
	incf	Time0i,	1		;; Time0i+1 -> Time0i  (new_Time0i)
	movf	Time0i,	0		;; -> W
	sublw	0x3b			;; (59-new_Time0i)
	btfss	STATUS,	C
	call	Countup2		;; C=0 negative
	return
	
Countup2:
	clrf	Time0i			;; set Time0i=0
	incf	Time1i,	1		;; Time1i+1 -> Time1i  (new_Time1i)
	movf	Time1i,	0		;; -> W
	sublw	0x3b			;; (59-new_Time1i)
	btfss	STATUS,	C
	clrf	Time1i			;; if C=0 , set Time1i=0
	return

;; --- Convert function Hex to BCD (T)
Diff_H2Tim:
	movwf	SubWork		; Store W -> SubWork
	sublw	0x3b		; 0x3b - W
	btfss	STATUS, C	; skip 0 or Positive
	retlw	0x24		; return with 0x24 -> W

	movfw	SubWork		; 
	sublw	0x31		; 0x31 - W
	btfss	STATUS, C	; skip 0 or Positive
	retlw	0x1e		; return with 0x1e -> W

	movfw	SubWork		; 
	sublw	0x27		; 0x27 - W
	btfss	STATUS, C	; skip 0 or Positive
	retlw	0x18		; return with 0x18 -> W

	movfw	SubWork		; 
	sublw	0x1d		; 0x1d - W
	btfss	STATUS, C	; skip 0 or Positive
	retlw	0x12		; return with 0x12 -> W

	movfw	SubWork		; 
	sublw	0x13		; 0x13 - W
	btfss	STATUS, C	; skip 0 or Positive
	retlw	0x0c		; return with 0x0c -> W

	movfw	SubWork		; 
	sublw	0x09		; 0x09 - W
	btfss	STATUS, C	; skip 0 or Positive
	retlw	0x06		; return with 0x06 -> W

	retlw	0x00		; return with 0x00 -> W

;******************************************************************************
Waitfor15:
;;--- Wait for 1.5msec 	
	movlw	15			;; 15 * 0.1mSec -> 1.5mSec
	movwf	Count
WaitLoop1:
	movlw	delay100us
	movwf	Count2
WaitLoop2:
	goto	WaitLoop4		; 2 * NOP
WaitLoop3:
	goto	WaitLoop3		; 2 * NOP
WaitLoop4:
	decfsz	Count2, 1
	goto	WaitLoop2

	decfsz	Count, 1
	goto	WaitLoop1
	return

;;***************************************************************************
ReserveISB:
;; --- Reserve an Issue Slave Break for the Next chance (T)
	bsf	ResPeriod, 0		;; set ResPeriod = 1

					;; June 5 2004 (T)
	movf	Display, 0		;; case Display = 3,
	movlw	0x03			;;
	btfsc	STATUS, Z		;;
	bsf	Int1secON, 0		;; set Int1secON = 1
	
	goto	ReceiveCommand		;;


;;***************************************************************************
Play_Stop:
;; --- Status change caused by Stop-Playing (T)
	call	RelSet_OFF		;; Relay OFF (disconnect) Apr. 07 2004

	call	Int_Reset

	clrf	Int1secON		;; clear Int1seON flag
	clrf	Display			;; clear Display
	clrf	ResPeriod		;; clear ResPeriod
	clrf	DropCheckCount		;; clear DropCheckCount
	clrf	NeedToTalk		;; No need to talk
	return

;;***************************************************************************
Waitfor100ms:
;; --- Wait for 100ms. (T)
	movlw	10			;; 10 * 10msec  -> 0.1Sec
	movwf	Count			;; "10" -> Count
WaitSlLoop0:
	movlw	100			;; 100 * 0.1mSec -> 10mSec
	movwf	Count2			;; "100" -> Count2
WaitSlLoop1:
	movlw	delay100us
	movwf	Count3
WaitSlLoop2:
	goto	WaitSlLoop3		; 2 * NOP
WaitSlLoop3:
	goto	WaitSlLoop4		; 2 * NOP
WaitSlLoop4:
	decfsz	Count3, 1
	goto	WaitSlLoop2
	decfsz	Count2, 1
	goto	WaitSlLoop1
	decfsz	Count, 1	
	goto	WaitSlLoop0
	return

;;***************************************************************************
;;---  Port RB3 is used as LED output  (T)
LED_ON:					;; LED Light ON
	bsf	STATUS, RP0		;; go bank1
	bcf	TRISB, LEDBit		;; Enable port as output
	bcf	STATUS, RP0		;; go bank0
	bsf	PORTB, LEDBit		;; set LED ON
	return

LED_OFF:				;; LED Light OFF
	bsf	STATUS, RP0		;; go bank1
	bcf	TRISB, LEDBit		;; Enable port as output
	bcf	STATUS, RP0		;; go bank0
	bcf	PORTB, LEDBit		;; set LED ON
	bsf	STATUS, RP0		;; go bank1
	bsf	TRISB, LEDBit		;; Tristate data pin
	bcf	STATUS, RP0		;; go bank0
	return


;;*******************************************************************
GetMyGroup:
;;--- Return GroupID with 0xD0/0x30 Alternatively. Feb. 21 2004  (T)
;;--- MD/CD switches by every TWO times.   Mar. 13 2004 (T)
;;           (ex.  MD,MD,CD,CD,MD,MD...)
;;--Note-  Initial value of GetMyG_Count if 0xff.
	incf	GetMyG_Count, 1
	btfss	GetMyG_Count, 1		;; check 1bit position
	retlw	0xD0			;; case Bit=0 return 0xD0 (MD)
	retlw	0x30			;; case Bit=1 return 0x30 (CD)


;;*******************************************************************
;; Solenoid Relay Control		;; Apr. 5 2004 (T)
;; Note- Twin turn latching type relay control function is not completed.
;;--- Control port RB5(Rel_Set) High/Low	;; Apr. 4 2004 (T)

RelSet_ON:				;; RelaySW ON(Make)
	bsf	STATUS, RP0		;; go bank1
	bcf	TRISB, Rel_Set		;; Enable port as output
	bcf	STATUS, RP0		;; go bank0

	bsf	PORTB, Rel_Set		;; ON
	nop
	nop
	bsf	PORTB, Rel_Set		;; ON
	return

RelSet_OFF:				;; RelaySW OFF(Break)
	bsf	STATUS, RP0		;; go bank1
	bcf	TRISB, Rel_Set		;; Enable port as output
	bcf	STATUS, RP0		;; go bank0

	bcf	PORTB, Rel_Set		;; OFF
	nop
	nop
	bcf	PORTB, Rel_Set		;; OFF

	bsf	STATUS, RP0		;; go bank1
	bsf	TRISB, Rel_Set		;; Tristate data pin
	bcf	STATUS, RP0		;; go bank0
	return

;;***************************************************************************
;; BusOnOut Set/Reset			;; Apr. 05 2004 (T)
SetBusOnOut:
	bsf	STATUS, RP0		;; go bank1
	bcf	TRISB, BusOnOut		;; Enable port as output
	bcf	STATUS, RP0		;; go bank0

	bsf	PORTB, BusOnOut		;; set BusOnOut=ON
	return

ResetBusOnOut:
	bsf	STATUS, RP0		;; go bank1
	bcf	TRISB, BusOnOut		;; Enable port as output
	bcf	STATUS, RP0		;; go bank0

	bcf	PORTB, BusOnOut		;; set BusOnOut=OFF

	bsf	STATUS, RP0		;; go bank1
	bsf	TRISB, BusOnOut		;; Tristate data pin
	bcf	STATUS, RP0		;; go bank0
	return

;;***************************************************************************
;; EEPROM Read/Write			;; May. 29 2004 (T)
Eeprom_Write:
eeprom_write:
	bsf	STATUS,RP0		;; Go Bank 1
	movwf	EEADR			;; W -> EEADR
	movf	ee_data,0		;; load Data -> W
	movwf	EEDATA			;; W -> EEDATA
	bsf	EECON1, WREN		;; set write enable
	bcf	INTCON, GIE		;; Disable GIE
	movlw	055h			;;-------------------
	movwf	EECON2			;;Special Procedure
	movlw	0aah			;;
	movwf	EECON2			;;-------------------
	bsf	EECON1,WR		;; exec EEPROM Write
eeprom_write2:
	nop
	btfsc	EECON1,WR		;; Check Flag goes Low
	goto	eeprom_write2		;; if Not completed, recheck.
	bsf	INTCON, GIE		;; if Finished , 
	bcf	STATUS,RP0		;; Go back to Bank 0
	return

Eeprom_Read:
eeprom_read:
	bsf	STATUS, RP0		;; go BANK1
	movwf	EEADR			;; W -> EEADR
	bsf	EECON1, RD		;; exec Read operation
	movf	EEDATA, 0 		;; result -> W
	bcf	STATUS, RP0		;; go back to BANK0
	return

;******************************************************************************
;; -- return one character from EEPROM
;; -- set start_ADDR into EepromChrAdd before call this.

RetEepromChr:
	movf	EepromChrAdd, 0		;; load EepromChrAdd -> W
	call	eeprom_read		;; read from EEPROM
	movwf	EeRet			;; read result -> store EeRet
	xorlw	0xff			;; check result = 0xff
	movlw	0x00			;; case 0xff, return 0x00 ->W
	btfss	STATUS, Z		;;
	movf	EeRet, 0		;; case not 0xff, return EeRet ->W
	incf	EepromChrAdd, 1		;; Next Addr
	return				;; return with W


;******************************************************************************
Int_Reset:
	bcf	INTCON, GIE			;; Disable GIE Interrupt
	nop
	bcf	INTCON, PEIE			;; Disable PEIE Interrupt
	nop					;;
	bcf	INTCON, INTE			;; Disable RB0/INT Interrupt
	bcf	T1CON, TMR1ON			;; set TMR1ON = Stop

	bsf	STATUS, RP0			;; go to Bank1
	bcf	PIE1, TMR1IE			;; Disable TMR1 Inerrupt
	bcf	STATUS, RP0			;; back to Bank0

	return

Int_Enaset:				;; Enable Interrpt (T)
	bsf	STATUS, RP0		;; go to Bank1
	bsf	PIE1, TMR1IE		;; Enable TMR1 Inerrupt
	bcf	STATUS, RP0		;; back to Bank0

	bsf	INTCON, PEIE		;; Enable PEIE Interrupt
	nop
	bsf	INTCON, GIE		;; Enable GIE Interrupt

	bsf	T1CON, TMR1ON		;; set TMR1ON = Start

	return

;;***************************************************************************
;; --------End of functions.asm

