;******************************************************************************
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    I (Simon Wood) can be contacted via email - unilink@mungewell.org
;    or via post - 14 Kirklands Close, Baildon, West Yorkshire, England.
;
;******************************************************************************
;	Additional Info - Release version  1.8T_02
;		Modified Version : GNUnilink for MDX-5V101R
;			"Released under the GPL license, 
;					(c) H "Turboh" Ishimoto, June 2004"
;					   Email: ishimoto@bea.hi-ho.ne.jp
;******************************************************************************

; Initializations of all the variables, called on power up on a bus reset.

SystemSetup:	
	IFDEF	DEBUG
;;--DEBUG settings -------
	movlw	-1
	movwf	DiskNum		;; set DiskNum = -2
				;; (Initial display DiskNum becomes "1")
	movlw	1
	movwf	TrackNum	;; set TrackNum = 1
;;------------------------
	ELSE
;;--Release settings -----
	movlw	1
	movwf	DiskNum		;; set DiskNum = 1 (T)
	movwf	TrackNum	;; set TrackNum = 1 (T)
;;------------------------
	ENDIF

;;---  default Disp mode - user selects
	movlw	DispCtrl_Init
	movwf	DispCtrl

;;--- DiscName Display ON/OFF flag (T)
	clrf	DiscNam_DSPL		


;;--- Initialize Magazine Slot Information (T)----------------------------
	clrf	SlotOfMag		;; for Working

	call	GetMag_Cd_EPROM		;; result -> W
	xorlw	0x09			
	movlw	0x05			;; case not 9, set 5
	btfsc	STATUS, Z
	movlw	0x09			;; case 9, set 9
	movwf	SlotOfMg_Cd

	call	GetMag_Md_EPROM		;; result -> W
	xorlw	0x09			
	movlw	0x05			;; case not 9, set 5
	btfsc	STATUS, Z
	movlw	0x09			;; case 9, set 9
	movwf	SlotOfMg_Md

	movlw	0x31			;;
	movwf	CurSource		;; default Current Source is CD_1
;;------------------------------------------------------------------------

;;-- Initialize EEPROM CD/MD Group Config Setting (0x12) -Jun 12 2004 ----
	movlw	0xff		;; EEPROM deta for default setting (0xff)
	movwf	ee_data		;; W -> EE_Data
	movlw	MyGroupEEADR	;; EEPROM(0x2112) data for GetMyGroup -> W
	call	Eeprom_Write	;; EEPROM write
;;------------------------------------------------------------------------

;;---  MDX-5V101R can display 10 chars on the first page of display (t)
;;      set "<AUX-IN>  "  by default.

	movlw	DiscNamADDR0		;; EEPROM:0x2120
	movwf	DiscNamADDR

	movlw	TrackNamADDR0		;; EEPROM:0x2140
	movwf	TrackNamADDR

;******************************************************************************
WarmBoot:
	call	ResetBusOnOut	;;  set BusOnOut= OFF (T)
	clrf	MyId 		; Not cleared when PIC-Wakeup (Resume) (T)
	clrf	MyBit		; Not cleared when PIC-Wakeup (Resume) (T)
	clrf	MyIdCont_Flag	;; MyId Appointing flag    Feb. 14 2004 (T)

	movlw	MyGroupEEADR	;; EEPROM(0x2112) data for GetMyGroup -> W
	call	Eeprom_Read	;; EEPROM read -> W
	xorlw	0x01		;; check value = 1 or not
	movlw	0xff		;; case NOT 0x01, set 0xff (next = MD mode)
	btfsc	STATUS, Z	;;
	movlw	0x01		;; case 0x01,  set 0x01 (next = CD mode)
	movwf	GetMyG_Count	;; set  GetMyG_Count

	movlw	0x80
	movwf	IdleState	;; Idle
	movlw	0xC0
	movwf	MD11State	;; Ejecting
	clrf	Ej_announceF	;; Not Announced yet

	IFDEF	DEBUG
;;--DEBUG setting --------
	incf	DiskNum,  1	;; countup by every re-initialize.
;;------------------------
	ENDIF

;******************************************************************************
Resume:				; PIC-Wakeup Entry (T)
	clrf	Sleeping	;; set dongle as waken-up.
	clrf	Display
	clrf	Time1
	clrf	Time0
	clrf	Timep
	clrf	Time0i
	clrf	Time1i
	clrf	ResPeriod
	clrf	DropCheckCount
	clrf	TrackNum_Dsp
	clrf	NeedToTalk		;; Apr. 19 2004 (T)
	clrf	DDir_Flag		;; Jun. 18 2004 (T)
	clrf	DDir_Num		;; Jun. 18 2004 (T)
	clrf	DSPL_Run		;; Jul. 31 2004 (T)
	clrf	Dsp_Field		;; Jul. 31 2004 (T)

;;--- for Appointing function
	clrf	Res_AppEnd_flag
	clrf	Appoint_Now		;; Apr. 07 2004 (T)
;;--- 

;;	movlw	CountupN
;;	movwf	CountupNwork		;; reset counter (T)

	bsf	PowDown, 0		;; set PowDown (1=down)

	IFDEF	NextNext
;;---------------------------
	clrf	NewDiscNumP
	clrf	NewDiscNumR
;;---------------------------
	ENDIF

	clrf	Ej_announceF		;; Not Announced yet

;;--- Port Configration.  ;; NOP are used to enforce stability (T)
	IFDEF	PIC_16
	bcf	STATUS, RP0
	nop
	IFDEF	PIC_16F62x
	bcf	STATUS, RP1
	ENDIF
	clrf	PORTA
	IFDEF	PIC_16F62x
	movlw	0x07			; Disable the Comparators
	movwf	CMCON
	ENDIF
	bsf	STATUS, RP0		; Configure Ports
	nop
	movlw	0xEF
	movwf	TRISA			; Port A

	movlw	0x03			;; Don't set 0x00 (T)
	movwf	TRISB			; Port B - even if not used

	movlw	0x0F			; Set up watchdog timer
	iorwf	OPTION_REG, 1
	clrwdt

	bcf	STATUS, RP0
	clrf	PORTA
	clrf	PORTB
	ENDIF

;;---					;; Feb. 21 2004  (T)
	IFDEF	CdMd_SW
	movlw	0x30			; Set up Grouping - Either CD or MD
	btfss	OUT_PORT, CdNotMdBit	; unconnected = high = CD
				;; Pull-Up is highly recommended
				;; unconnected causes unstable behavior  (T)
	movlw	0xD0
	movwf	MyGroup
	ENDIF


;;--- TMR1 interrupt setting    	May 22 2004  (T) 
Int1sec_Config:
;;*-----------------------------------------*
;;*	T1CKPS1=1
;;*	T1CKPS0=1
;;*	T1OSCEN=0
;;*	T1SYNC=DC=0
;;*	TMR1CS=0
;;*	TMR1ON=0
;;*       ===>  T1CON=B'00110000 = 0x30
;;*-----------------------------------------*
	movlw	0x30
	movwf	T1CON			;; 

	bsf	STATUS, RP0		;; go to Bank1
	movlw	0xd7			;; Pre-scaler setting 
	movwf	OPTION_REG
	bcf	STATUS, RP0		;; back to Bank0

	clrf	INTCON			;; Disable global interrupt
	clrf	Int1secON		;; clear Int1secON flag
	clrf	TMR1H			;; 
	clrf	TMR1L			;; 

;;--- Set BusOn_Out
	movf	MyId, 1			;; Test if we've got an ID
	btfss	STATUS, Z
					;; (case MyId is NOT null)
	call	SetBusOnOut		;; set BusOnOut=ON  Apr. 11 2004 (T)

;;--- Turn the LED light ON
	call	LED_ON			;; Feb. 14 2004 (T)

;;--- Relay OFF (disconnect)
	call	RelSet_OFF		;; Apr. 07 2004 (T)

;;--- Goto Main Loop
	goto	ReceiveBreak

;;***********************************************************************
;; --------End of setup.asm
